var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["cilantro/define/criteriamanager","cilantro/define/conceptmanager"],function(a,b){var c;return c=function(){function c(){this.activate=__bind(this.activate,this),c.__super__.constructor.apply(this,arguments)}return __extends(c,Backbone.View),c.prototype.el="#plugin-panel",c.prototype.initialize=function(){return App.hub.subscribe("concept/active",this.activate)},c.prototype.activate=function(c){var d;return d=a.retrieveCriteriaDS(c.id),b.isConceptLoaded(c.id)?b.show({id:c.id},d):c.fetch({beforeSend:__bind(function(){return this.el.block()},this),complete:__bind(function(){return this.el.unblock()},this),success:function(){return b.show(c.get("viewset"),d)}})},c}(),{ConceptInterfaceView:c}})