require(["cilantro/types/report/main","cilantro/report/table","cilantro/report/columns"],function(a,b,c){var d;return d=new a.Models.Session,App.hub.subscribe("session/idle",function(){return d.stopPolling()}),App.hub.subscribe("session/resume",function(){return d.startPolling()}),$(function(){var e,f,g,h;return e=new a.Views.Editor,f=new a.Views.Name({model:d}),g=new a.Messages.UnsavedReport({model:d}),d.fetch(),c.init(),b.init(),h=$("#export-data"),h.bind("click",function(){return h.attr("disabled",!0),window.location=App.endpoints.session.report+"?data=1&format=csv",setTimeout(function(){return h.attr("disabled",!1)},5e3),!1})})})