var __bind=function(a,b){return function(){return a.apply(b,arguments)}},__hasProp=Object.prototype.hasOwnProperty,__extends=function(a,b){function d(){this.constructor=a}for(var c in b)__hasProp.call(b,c)&&(a[c]=b[c]);return d.prototype=b.prototype,a.prototype=new d,a.__super__=b.prototype,a};define(["common/utils"],function(a){var b,c,d;return b={pollInterval:1e4,initialize:function(){return this.startPolling()},startPolling:function(){return this._pollInterval=setInterval(__bind(function(){return this.poll()},this),this.pollInterval)},stopPolling:function(){return clearTimeout(this._pollInterval)}},d=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,Backbone.Model),a.prototype.poll=function(){return this.fetch()},a}(),c=function(){function a(){a.__super__.constructor.apply(this,arguments)}return __extends(a,Backbone.Collection),a.prototype.poll=function(){return this.fetch({update:!0})},a}(),a.include(d,b),a.include(c,b),{Model:d,Collection:c}})